// Flash driver for nRF52840 | gat 11/04/23
#include "Flash.h"
#include "nrf52840.h"
#include "nrf52840_bitfields.h"

enum
{
	ReadOnly,
	Write,
	Erase
}Mode;

static inline void wait_for_flash_ready(void)
{
  while (NRF_NVMC->READY == NVMC_READY_READY_Busy) {;}
}

static void LL_write_byte(uint32_t address, uint8_t value)
{
    uint32_t byte_shift = address & (uint32_t)0x03;
    uint32_t address32 = address & ~byte_shift; // Address to the word this byte is in.
    uint32_t value32 = (*(uint32_t*)address32 & ~((uint32_t)0xFF << (byte_shift << (uint32_t)3)));
    value32 = value32 + ((uint32_t)value << (byte_shift << 3));

    NRF_NVMC->CONFIG = Write;
    __ISB();
    __DSB();

    *(uint32_t*)address32 = value32;
    wait_for_flash_ready();

    NRF_NVMC->CONFIG = ReadOnly;
    __ISB();
    __DSB();
}

void Flash_Write(unsigned int address, unsigned int size, void* ptr)
{
	unsigned char* data = (unsigned char*) ptr;
	Flash_Erase(address);
	for(int i = 0; i < size; i++)
	{
		LL_write_byte(address + i, data[i]);
	}
}

void Flash_Read(unsigned int address, unsigned int size, void* ptr)
{
	unsigned char* data = (unsigned char*)ptr;
	for(unsigned int i = 0; i < size; i++)
		data[i] = *((unsigned char *)(address + i));
}

void Flash_Erase(unsigned int page_address)
{
	wait_for_flash_ready();
	NRF_NVMC->CONFIG = Erase;
	wait_for_flash_ready();
	NRF_NVMC->ERASEPAGE = page_address;
	wait_for_flash_ready();
	NRF_NVMC->CONFIG = ReadOnly;
	wait_for_flash_ready();
}
