// GPIO driver for nRF52840 | gat 11/04/23
#ifndef _GPIO_H_
#define _GPIO_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "nrf52840.h"
#include "nrf52840_bitfields.h"

/* configure as input, input buffer connected and pull disabled, standard 0 standard 1 */
#define io_dir_in(port, pin)				NRF_P##port->PIN_CNF[pin] = (GPIO_PIN_CNF_DIR_Input << GPIO_PIN_CNF_DIR_Pos) + (GPIO_PIN_CNF_INPUT_Connect << GPIO_PIN_CNF_INPUT_Pos)
// configure as unused pin
#define io_unused_(port, pin)				NRF_P##port->PIN_CNF[pin] = (GPIO_PIN_CNF_INPUT_Disconnect << GPIO_PIN_CNF_INPUT_Pos)
/* configure as output, input buffer disconnected and pull disabled, standard 0 standard 1 */
#define io_dir_out(port, pin)				NRF_P##port->PIN_CNF[pin] = (GPIO_PIN_CNF_DIR_Output << GPIO_PIN_CNF_DIR_Pos) + (GPIO_PIN_CNF_INPUT_Disconnect << GPIO_PIN_CNF_INPUT_Pos)

/* direction get 	1:out 	0:in */
#define io_dir_get(port, pin)				((NRF_P##port->DIR >> pin) & 1UL)

/* connect input buffer (for SPI Clock) */
#define io_buf_connect(port, pin)		NRF_P##port->PIN_CNF[pin] &= ~(GPIO_PIN_CNF_INPUT_Disconnect << GPIO_PIN_CNF_INPUT_Pos)

/* configure as input, input buffer connected and pull up abled, standard 0 standard 1 */
#define io_pull_up(port, pin)				NRF_P##port->PIN_CNF[pin] |= (GPIO_PIN_CNF_PULL_Pullup << GPIO_PIN_CNF_PULL_Pos)

/* configure as input, input buffer connected and pull down abled, standard 0 standard 1 */
#define io_pull_down(port, pin)			NRF_P##port->PIN_CNF[pin] |= (GPIO_PIN_CNF_PULL_Pulldown << GPIO_PIN_CNF_PULL_Pos)

/* configure as strong 0 strong 1 */
#define io_drive_(port, pin)			NRF_P##port->PIN_CNF[pin] |= (GPIO_PIN_CNF_DRIVE_H0H1 << GPIO_PIN_CNF_DRIVE_Pos)

/* output set */
#define io_set_(port, pin)					NRF_P##port->OUTSET = (1U << pin)

/* output clear */
#define io_clr_(port, pin)					NRF_P##port->OUTCLR = 1U << pin

/* output toggle */
#define io_tog_(port, pin)					NRF_P##port->OUT ^= (1U << pin)

/* input status read */
#define io_get_(port, pin)					(io_dir_get(port, pin))?io_get_out(port, pin):io_get_in(port, pin)

/* input status read */
#define io_get_out(port, pin)					((NRF_P##port->OUT >> pin) & 1U)

/* input status read */
#define io_get_in(port, pin)					((NRF_P##port->IN >> pin) & 1U)

/* configure as input for events generating */
#define iote_dir_in(port, pin)			io_dir_in(port, pin); NRF_P##port->PIN_CNF[pin] = (GPIO_PIN_CNF_SENSE_High<<GPIO_PIN_CNF_SENSE_Pos)

#define io_in(port, pin) io_dir_in(port, pin) // Tweak to add an extra step of expansion
#define io_out(port, pin) io_dir_out(port, pin) // Tweak to add an extra step of expansion
#define iote_in(port, pin) iote_dir_in(port, pin) // Tweak to add an extra step of expansion
#define io_buf(port, pin) io_buf_connect(port, pin) // Tweak to add an extra step of expansion
#define io_pullup(port, pin) io_pull_up(port, pin) // Tweak to add an extra step of expansion
#define io_pulldown(port, pin) io_pull_down(port, pin) // Tweak to add an extra step of expansion
#define io_set(port, pin) io_set_(port, pin) // Tweak to add an extra step of expansion
#define io_get(port, pin) io_get_(port, pin) // Tweak to add an extra step of expansion
#define io_clr(port, pin) io_clr_(port, pin) // Tweak to add an extra step of expansion
#define io_tog(port, pin) io_tog_(port, pin) // Tweak to add an extra step of expansion
#define io_unused(port, pin)	io_unused_(port, pin) // Tweak to add an extra step of expansion
#define io_drive(port, pin) io_drive_(port, pin) // Tweak to add an extra step of expansion

void GPIO_Setup(void);

#ifdef __cplusplus
}
#endif

#endif //_GPIO_H_
